function [T_computed] = ShapeFit(V)

% This function solves the ShapeFit program
%
%      MIN_{t_i}  \sum_{ij in E} ||P_{vij^perp} (t_i - t_j)||_2
%      SUBJECT TO \sum_i t_i = 0 AND \sum_{ij in E} <t_i-t_j, vij> = 1
%


% Input:
% V - d x n x n - V(:,i,j) is the possibly noisy and corrupted direction of T0(:,i) - T0(:,j)
%                 V(:,i,j) is zero for i>j, as entries could be found by antisymmetry
%                 V(:,i,i) is zero for all i, as entries are unneeded and ill-defined
%                 V(:,i,j) is nonzero for values of i<j that correspond to measurements given by E
%
% Output:
%
% T_computed - d x n - Estimated locations based on ShapeFit.  
%                      Values are NaN if any problems occured in SDPT3 Solver

[d,n,~] = size(V);
is_present= @(i,j) any(V(:,i,j)); % Determines if pair (i,j) corresponds to an observation

%% Form objective and constraint for YALMIP/SDPT3
T = sdpvar(d,n, 'full');
F = [T*ones(n, 1) == 0];
L = 0;

obj = 0;
for i=1:n
    for j=i+1:n
        if is_present(i,j)
            L = L + (T(:,i) - T(:, j))'*V(:,i,j);
            obj = obj + norm( (eye(d)-V(:,i,j)*V(:,i,j)' / norm(V(:,i,j))^2) * (T(:,i) - T(:,j) ) );
        end
    end
end
F = [F, L == 1];


output = solvesdp(F, obj)

%% Format output, returning NaNs if there are any problems
if output.problem
    T_computed = NaN*ones(size(T));
else
    T_computed = double(T);
end


end

