function [V] = make_observations(G, C, T0, sigma)

% This function generates the direction observations on the locations T0
% in accordance with the graph of measurements G and the (sub)graph of corruptions C
% The measurements are generated by
%
% Let zij ~ Unif(S^(d-1)) be i.i.d.
% Let vij = normalize(normalize(t0_i-t0_j) + sigma zij) for ij in G\C
% Let vij = zij for ij in C
%
% normalize(x) = x / ||x||_2 

% Input:
%
% G  - n x n - upper triangular part of adjacency matrix of measurement graph
% C  - n x n - upper triangular part of adjacency matrix of corruption grap
%              all corrupted edges must also belong to G
% T0 - d x n - set of points to recover; each column is a point in R^d
% sigma - noise parameter

% Output:
% V - d x n x n - V(:,i,j) is the direction of T0(:,i) - T0(:,j) subject to noise and corruption
%                 only nonzero for j>i because i<j entries are could be found by antisymmetry

[d,n] = size(T0);
if nargin < 4
    sigma = 0;
end

V = zeros(d,n,n);

for i = 1:n
    for j = i+1:n
        if C(i,j)
            % If edge is `corrupted', observed direction is random
            V(:,i,j) = normalize(randn(d,1));  
        elseif G(i,j)
            % If edge is `uncorrupted', observed direction is noisy version
            % of direction of T0(:,i) relative to T0(:,j)
            V(:,i,j) = normalize(normalize(T0(:,i) - T0(:,j)) + sigma * normalize(randn(d,1)));
        end
    end
end

return


function y = normalize(x)
y = x / norm(x);    % l2 normalization for vectors
return
